#!/bin/bash
VERSION=0
MINOR_VERSION=4

# cwd=`dirname $0`
cwd=${PWD}

#configuration
NEED_EXTRACT_TOOL=
NEED_MAKE_TARGET=1
NEED_MIGRATE_TO_SD=1

#configuration for Silicon & board related
SYSFW_SOC=j721e
#SYSFW_SOC=j721e_sr1_1
SOC_HS_TYPE=hs
# SOC_HS_TYPE=hsp

#configuration for intermidiate usage
BOOT_SPL_MODE=1
#this is BOOT SBL MODE
#BOOT_SPL_MODE=0
# FULL_BINARY_SIGNED=0
FULL_BINARY_SIGNED=1

#common usage
RCORE_FLAG=r5
ACORE_FLAG=a72
SPL_FLAG=spl

#Silicon & board related
SOC=j721e
BOARD=j721e_evm

#SDK related, currently base on SDK8.6
SDK_VERSION=0806
RELATIVE_ARM_LLVM_PATH=ti-cgt-armllvm_1.3.0.LTS
RELATIVE_CGT_C6000_PATH=ti-cgt-c6000_8.3.7
RELATIVE_CGT_C7x_PATH=ti-cgt-c7000_3.1.0.LTS
K3_IMAGE_GEN=k3-image-gen-2022.01
CORE_SECDEV=core-secdev-k3
RELATIVE_PDK_PATH=pdk_jacinto_08_06_00_31
RELATIVE_K3_IMAGE_GEN_PATH=board-support/${K3_IMAGE_GEN}
RELATIVE_CORE_SECDEV_PATH=board-support/${CORE_SECDEV}
RELATIVE_UBOOT_PATH=board-support/u-boot-2021.01+gitAUTOINC+62a9e51344-g62a9e51344
RELATIVE_KERNEL_PATH=board-support/linux-5.10.162+gitAUTOINC+76b3e88d56-g76b3e88d56
RELATIVE_U_BOOT_HS_BUILD_PATH=board-support/u-boot_hs_build
RELATIVE_PREBUILT_PATH=board-support/prebuilt-images
RELATIVE_SECURE_BINARY_IMAGE_PATH=${SOC}_${SDK_VERSION}/${CORE_SECDEV}/scripts/secure-binary-image.sh

#intermidiate usage
EXTRACT_TOOL_FLAG=0
MAKE_TARGET_FLAG=0
MIGRATE_TO_SD_FLAG=0
KEY_FILE=
LINUX_SDK=
OUTPUT_PATH=
PREBUILT_PATH=
UBOOTRELEASE=
SOC_UID=

function print_debug_info {
    echo "print for debug:"
    echo "pre-configuration:"
    echo "NEED_EXTRACT_TOOL:${NEED_EXTRACT_TOOL}:"
    echo "NEED_MAKE_TARGET:${NEED_MAKE_TARGET}:"
    echo "NEED_MIGRATE_TO_SD:${NEED_MIGRATE_TO_SD}:"
    echo "Silicon & board related:"
    echo "SOC:${SOC}:"
    echo "BOARD:${BOARD}:"
    echo "SYSFW_SOC:${SYSFW_SOC}:"
    echo "SOC_HS_TYPE:${SOC_HS_TYPE}:"
    echo "common usage:"
    echo "RCORE_FLAG:${RCORE_FLAG}:"
    echo "ACORE_FLAG:${ACORE_FLAG}:"
    echo "SPL_FLAG:${SPL_FLAG}:"
    echo "SDK related:"
    echo "SDK_VERSION:${SDK_VERSION}:"
    echo "RELATIVE_ARM_LLVM_PATH:${RELATIVE_ARM_LLVM_PATH}:"
    echo "RELATIVE_CGT_C6000_PATH:${RELATIVE_CGT_C6000_PATH}:"
    echo "RELATIVE_CGT_C7x_PATH:${RELATIVE_CGT_C7x_PATH}:"
    echo "K3_IMAGE_GEN:${K3_IMAGE_GEN}:"
    echo "CORE_SECDEV:${CORE_SECDEV}:"
    echo "RELATIVE_PDK_PATH:${RELATIVE_PDK_PATH}:"
    echo "RELATIVE_K3_IMAGE_GEN_PATH:${RELATIVE_K3_IMAGE_GEN_PATH}:"
    echo "RELATIVE_CORE_SECDEV_PATH:${RELATIVE_CORE_SECDEV_PATH}:"
    echo "RELATIVE_UBOOT_PATH:${RELATIVE_UBOOT_PATH}:"
    echo "RELATIVE_KERNEL_PATH:${RELATIVE_KERNEL_PATH}:"
    echo "RELATIVE_U_BOOT_HS_BUILD_PATH:${RELATIVE_U_BOOT_HS_BUILD_PATH}:"
    echo "RELATIVE_PREBUILT_PATH:${RELATIVE_PREBUILT_PATH}:"
    echo "RELATIVE_SECURE_BINARY_IMAGE_PATH:${RELATIVE_SECURE_BINARY_IMAGE_PATH}:"
    echo "intermidiate usage:"
    echo "EXTRACT_TOOL_FLAG:${EXTRACT_TOOL_FLAG}:"
    echo "MAKE_TARGET_FLAG:${MAKE_TARGET_FLAG}:"
    echo "MIGRATE_TO_SD_FLAG:${MIGRATE_TO_SD_FLAG}:"
    echo "FULL_BINARY_SIGNED:${FULL_BINARY_SIGNED}:"
    echo "BOOT_SPL_MODE:${BOOT_SPL_MODE}:"
    echo "KEY_FILE:${KEY_FILE}:"
    echo "LINUX_SDK:${LINUX_SDK}:"
    echo "OUTPUT_PATH:${OUTPUT_PATH}:"
    echo "PREBUILT_PATH:${PREBUILT_PATH}:"
    echo "UBOOTRELEASE:${UBOOTRELEASE}:"
    echo "SOC_UID:${SOC_UID}:"
    echo "end"
}




function check_if_num {
    echo "$*"
    if [[ ! "$*" =~ ^[0-9]+$ ]]
    then
        echo "input is not number"
        exit
    fi 
}

function check_if_num_char {
    echo "$*"
    if [[ ! "$*" =~ ^[a-zA-Z0-9]+$ ]]
    then
        echo "input is not only number and char"
        exit
    fi 
}


function input_code_yn {
    read -p "$1(y[es]/n[o]):" temp
    if [ ! -n "${temp}" ]; then
       temp="n"
    fi
    if [ "${temp}" = "n" ]; then
      return 0
      else
      return 1
    fi
}
function input_code_pb {
    read -p "$1(p[re]/b[uild]/n[o]):" temp
    if [ ! -n "${temp}" ]; then
       temp="n"
    fi
    if [ "${temp}" = "n" ]; then
      return 0
      else
        if [ "${temp}" = "b" ]; then
            return 1
        else
            if [ "${temp}" = "p" ]; then
                return 2
            else
                return 0
            fi
        fi
    fi
}

function check_local_file_exist {
    if [ ! -f $* ]
    then
        echo "file, $*, not exist"
        return 1
    fi
    echo "file, $*, exist"
    return 0
}

function check_local_file_exist_then_exit {
    check_local_file_exist $*
    if [ ! $? -eq 0 ]
    then
        echo "$* not exist, exit"
        exit
    fi
}

function check_local_folder_exist {
    if [[ -e $* && -d $* ]]
    then
        echo "folder, $*, exist"
        return 0
    fi
    echo "folder, $*, not exist"
    return 1
}

function check_local_folder_exist_then_exit {
    check_local_folder_exist $*
    if [ ! $? -eq 0 ]
    then
        echo "$* not exist, exit"
        exit
    fi
}

function set_linux_sdk {
    read -p "please set LINUX SDK path, example:[/opt/1Twork/repository/ti-processor-sdk-linux-j7-evm-08_06_00_11]:" LINUX_SDK
    if [ ! -n "$LINUX_SDK" ]; then
        echo "LINUX_SDK path [${LINUX_SDK}] is null, use default path"
        if [[ -e ${cwd}/ti-processor-sdk-linux-j7-evm-08_06_00_11 && -d ${cwd}/ti-processor-sdk-linux-j7-evm-08_06_00_11 ]]; then
            LINUX_SDK=${cwd}/ti-processor-sdk-linux-j7-evm-08_06_00_11
        else
            echo "default, ${cwd}/ti-processor-sdk-linux-j7-evm-08_06_00_11, is invalid, exit"
            exit
        fi
    fi
}

function set_rtos_sdk {
    read -p "please set RTOS SDK path, example:[/opt/1Twork/repository/ti-processor-sdk-rtos-j721e-evm-08_06_00_12]:" RTOS_SDK
    if [ ! -n "$RTOS_SDK" ]; then
        echo "RTOS_SDK path [${RTOS_SDK}] is null, use default path"
        if [[ -e ${cwd}/ti-processor-sdk-rtos-j721e-evm-08_06_00_12 && -d ${cwd}/ti-processor-sdk-rtos-j721e-evm-08_06_00_12 ]]; then
            RTOS_SDK=${cwd}/ti-processor-sdk-rtos-j721e-evm-08_06_00_12
        else
            echo "default, ${cwd}/ti-processor-sdk-rtos-j721e-evm-08_06_00_12, is invalid, exit"
            exit
        fi
    fi
}


function set_output_path {
    read -p "please set output path:" OUTPUT_PATH
    if [ ! -n "$OUTPUT_PATH" ]; then
        echo "output path [${OUTPUT_PATH}] is invalid, set to current path"
        mkdir -p $cwd/${SOC}_${SDK_VERSION}/out
        OUTPUT_PATH=$cwd/${SOC}_${SDK_VERSION}/out
    fi
}


function set_prebuilt_path {
    if [[ ! -e ${cwd}/${SOC}_${SDK_VERSION}/prebuilt-images || ! -d ${cwd}/${SOC}_${SDK_VERSION}/prebuilt-images ]]; then
        mkdir -p $cwd/${SOC}_${SDK_VERSION}/prebuilt-images
        if [ $? -eq 1 ]; then
        exit
        fi
    fi
    PREBUILT_PATH=$cwd/${SOC}_${SDK_VERSION}/prebuilt-images
}

function set_uid {
    read -p "please set SOC UID:" SOC_UID
    check_if_num_char ${SOC_UID}
    if [ ${#SOC_UID} -ne 64 ]; then
        echo "SOC_UID [${SOC_UID}]: is invalid"
        exit
    fi
}


# First version edit by Biao
# This script is use for custoemr need HS silicon CPR(customer production return) 
# Only for spl boot binary need signed by customer
# Ask for directories and file paths
echo "Welcome to use this script to generate signed binary use customer priviate key that need send to TI for the CRP bench test unlock the silicon"
echo "Version:${VERSION}.${MINOR_VERSION}"

if [ ${SOC} = "j721e" ]; then
    echo "currently this tool only support TDA4VM..."
fi

if [ ! ${SDK_VERSION} -eq 806 ]
then
echo "sdk version 0806 is verified."
echo "sdk version ${SDK_VERSION} is invalid, exit..."
fi

if [ -n "$NEED_EXTRACT_TOOL" ]; then
    if [ $NEED_EXTRACT_TOOL -eq 1 ]; then
        EXTRACT_TOOL_FLAG=1
    else    
        EXTRACT_TOOL_FLAG=0
    fi
else
    EXTRACT_TOOL_FLAG=0
fi

if [ -n "$NEED_MAKE_TARGET" ]; then
    if [ $NEED_MAKE_TARGET -eq 1 ]; then
        MAKE_TARGET_FLAG=1
    else    
        MAKE_TARGET_FLAG=0
    fi
else
    MAKE_TARGET_FLAG=0
fi


if [ -n "$NEED_MIGRATE_TO_SD" ]; then
    if [ $NEED_MIGRATE_TO_SD -eq 1 ]; then
        MIGRATE_TO_SD_FLAG=1
    else    
        MIGRATE_TO_SD_FLAG=0
    fi
else
    MIGRATE_TO_SD_FLAG=0
fi

if [ ${BOOT_SPL_MODE} -eq 1 ]; then
    echo "SPL boot mode"
else
    echo "SBL boot mode"
fi
#prepare environment

function SPL_extract_prepare {
    echo "prepare environment to extract this tool from LINUX SDK"
    set_linux_sdk
    set_rtos_sdk
    set_prebuilt_path
    #set_output_path

    check_local_folder_exist_then_exit ${LINUX_SDK}/${RELATIVE_K3_IMAGE_GEN_PATH}
    check_local_folder_exist_then_exit ${LINUX_SDK}/${RELATIVE_CORE_SECDEV_PATH}

    print_debug_info
}
function SBL_extract_prepare {
    echo "prepare environment to extract this tool from RTOS SDK"
    set_linux_sdk
    set_rtos_sdk
    set_prebuilt_path
    #set_output_path

    check_local_folder_exist_then_exit ${RTOS_SDK}/${RELATIVE_PDK_PATH}

    print_debug_info
}


if [ ${EXTRACT_TOOL_FLAG} -eq 1 ]; then
    if [ ${BOOT_SPL_MODE} -eq 1 ]; then
        rm -rf $cwd/${SOC}_${SDK_VERSION}
        mkdir -p $cwd/${SOC}_${SDK_VERSION}
        SPL_extract_prepare
    else
        rm -rf $cwd/${SOC}_${SDK_VERSION}
        mkdir -p $cwd/${SOC}_${SDK_VERSION}
        SBL_extract_prepare
    fi
fi

function SPL_extract_do {
    echo "#######################################################################"
    echo "try to extract this tool from LINUX SDK"


if [ ! ${FULL_BINARY_SIGNED} -eq 1 ]; then
    echo "make sure patch/${SDK_VERSION}/0001-HS_main_domain_nosign.patch has been applied to LINUX SDK"
    read -p "press ENTER to continue..."
fi

    cp -rf ${LINUX_SDK}/${RELATIVE_K3_IMAGE_GEN_PATH} $cwd/${SOC}_${SDK_VERSION}
    cp -rf ${LINUX_SDK}/${RELATIVE_CORE_SECDEV_PATH} $cwd/${SOC}_${SDK_VERSION}

    cp ${LINUX_SDK}/${RELATIVE_UBOOT_PATH}/tools/k3_gen_x509_cert.sh $cwd/${SOC}_${SDK_VERSION}
    cp ${LINUX_SDK}/${RELATIVE_UBOOT_PATH}/tools/k3_fit_atf.sh $cwd/${SOC}_${SDK_VERSION}

    #try to bring out the staff to build tiboot3.bin and sysfw.itb
    make -C ${LINUX_SDK} u-boot-r5 SOC_TYPE=hs SYSFW_SOC=${SYSFW_SOC}
    cp ${LINUX_SDK}/${RELATIVE_U_BOOT_HS_BUILD_PATH}/${RCORE_FLAG}/spl/u-boot-spl.bin ${PREBUILT_PATH}/${RCORE_FLAG}_${SPL_FLAG}_u-boot-spl.bin
    cp ${LINUX_SDK}/${RELATIVE_K3_IMAGE_GEN_PATH}/out/soc/j721e/evm/pm-cfg.bin ${PREBUILT_PATH}
    cp ${LINUX_SDK}/${RELATIVE_K3_IMAGE_GEN_PATH}/out/soc/j721e/evm/sec-cfg.bin ${PREBUILT_PATH}
    cp ${LINUX_SDK}/${RELATIVE_K3_IMAGE_GEN_PATH}/out/soc/j721e/evm/board-cfg.bin ${PREBUILT_PATH}
    cp ${LINUX_SDK}/${RELATIVE_K3_IMAGE_GEN_PATH}/out/soc/j721e/evm/rm-cfg.bin ${PREBUILT_PATH}
    cp ${LINUX_SDK}/${RELATIVE_PREBUILT_PATH}/ti-fs-firmware-${SYSFW_SOC}-hs-cert.bin ${PREBUILT_PATH}
    cp ${LINUX_SDK}/${RELATIVE_PREBUILT_PATH}/ti-fs-firmware-${SYSFW_SOC}-hs-enc.bin ${PREBUILT_PATH}
    cp ${LINUX_SDK}/${RELATIVE_U_BOOT_HS_BUILD_PATH}/${RCORE_FLAG}/tools/mkimage ${PREBUILT_PATH}/${RCORE_FLAG}_${SPL_FLAG}_mkimage

    if [ ${FULL_BINARY_SIGNED} -eq 1 ]; then
        #try to bring out the staff to build tispl.bin and u-boot.img
        make -C ${LINUX_SDK} u-boot-a72 SOC_TYPE=hs
        cp ${LINUX_SDK}/${RELATIVE_U_BOOT_HS_BUILD_PATH}/${ACORE_FLAG}/spl/dts/k3-j721e-common-proc-board.dtb ${PREBUILT_PATH}/${ACORE_FLAG}_${SPL_FLAG}_k3-j721e-common-proc-board.dtb
        cp ${LINUX_SDK}/${RELATIVE_U_BOOT_HS_BUILD_PATH}/${ACORE_FLAG}/spl/dts/k3-j721e-tps65917-proc-board.dtb ${PREBUILT_PATH}/${ACORE_FLAG}_${SPL_FLAG}_k3-j721e-tps65917-proc-board.dtb
        cp ${LINUX_SDK}/${RELATIVE_U_BOOT_HS_BUILD_PATH}/${ACORE_FLAG}/spl/dts/k3-j721e-sk.dtb ${PREBUILT_PATH}/${ACORE_FLAG}_${SPL_FLAG}_k3-j721e-sk.dtb
        cp ${LINUX_SDK}/${RELATIVE_U_BOOT_HS_BUILD_PATH}/${ACORE_FLAG}/spl/u-boot-spl-nodtb.bin ${PREBUILT_PATH}/${ACORE_FLAG}_${SPL_FLAG}_u-boot-spl-nodtb.bin

        cp ${LINUX_SDK}/${RELATIVE_U_BOOT_HS_BUILD_PATH}/${ACORE_FLAG}/arch/arm/dts/k3-j721e-common-proc-board.dtb ${PREBUILT_PATH}/${ACORE_FLAG}_k3-j721e-common-proc-board.dtb
        cp ${LINUX_SDK}/${RELATIVE_U_BOOT_HS_BUILD_PATH}/${ACORE_FLAG}/arch/arm/dts/k3-j721e-tps65917-proc-board.dtb ${PREBUILT_PATH}/${ACORE_FLAG}_k3-j721e-tps65917-proc-board.dtb
        cp ${LINUX_SDK}/${RELATIVE_U_BOOT_HS_BUILD_PATH}/${ACORE_FLAG}/arch/arm/dts/k3-j721e-sk.dtb ${PREBUILT_PATH}/${ACORE_FLAG}_k3-j721e-sk.dtb
        cp ${LINUX_SDK}/${RELATIVE_U_BOOT_HS_BUILD_PATH}/${ACORE_FLAG}/u-boot-nodtb.bin ${PREBUILT_PATH}/${ACORE_FLAG}_u-boot-nodtb.bin

        cp ${LINUX_SDK}/${RELATIVE_U_BOOT_HS_BUILD_PATH}/${ACORE_FLAG}/tools/mkimage ${PREBUILT_PATH}/${ACORE_FLAG}_mkimage

        cp ${LINUX_SDK}/${RELATIVE_PREBUILT_PATH}/bl31.bin ${PREBUILT_PATH}
        cp ${LINUX_SDK}/${RELATIVE_PREBUILT_PATH}/bl32.bin ${PREBUILT_PATH}
        cp ${LINUX_SDK}/${RELATIVE_PREBUILT_PATH}/ipc_echo_testb_mcu1_0_release_strip.xer5f ${PREBUILT_PATH}
    fi

    if [ ${FULL_BINARY_SIGNED} -eq 1 ]; then
        #this is to retrieve uboot version.
        UBOOTRELEASE=$(cat ${LINUX_SDK}/${RELATIVE_U_BOOT_HS_BUILD_PATH}/${ACORE_FLAG}/include/config/uboot.release 2> /dev/null)
        UBOOTRELEASE=${UBOOTRELEASE/\"\"/}
        temp_release="U-Boot ${UBOOTRELEASE} for ${SOC} board"
        echo ${temp_release}
        touch ${PREBUILT_PATH}/temp.txt
        echo ${temp_release} > ${PREBUILT_PATH}/temp.txt
    fi

    if [ ${FULL_BINARY_SIGNED} -eq 1 ]; then
        #try to bring out the staff to build kernel.
        make -C ${LINUX_SDK} linux
        mkdir -p ${PREBUILT_PATH}/kernel/ti
        for DTB in      ti/k3-j721e-common-proc-board.dtb     ti/k3-j721e-proc-board-tps65917.dtb     ti/k3-j721e-common-proc-board-infotainment.dtbo     ti/k3-j721e-cpb-csi2-ov5640.dtbo     ti/k3-j721e-fpdlink-cpb-fusion.dtbo     ti/k3-j721e-fpdlink-imx390-cm-0-0.dtbo     ti/k3-j721e-fpdlink-imx390-cm-0-1.dtbo     ti/k3-j721e-fpdlink-imx390-cm-0-2.dtbo     ti/k3-j721e-fpdlink-imx390-cm-0-3.dtbo     ti/k3-j721e-fpdlink-imx390-cm-1-0.dtbo     ti/k3-j721e-fpdlink-imx390-cm-1-1.dtbo     ti/k3-j721e-fpdlink-imx390-cm-1-2.dtbo     ti/k3-j721e-fpdlink-imx390-cm-1-3.dtbo     ti/k3-j721e-fpdlink-imx390-rcm-0-0.dtbo     ti/k3-j721e-fpdlink-imx390-rcm-0-1.dtbo     ti/k3-j721e-fpdlink-imx390-rcm-0-2.dtbo     ti/k3-j721e-fpdlink-imx390-rcm-0-3.dtbo     ti/k3-j721e-fpdlink-imx390-rcm-1-0.dtbo     ti/k3-j721e-fpdlink-imx390-rcm-1-1.dtbo     ti/k3-j721e-fpdlink-imx390-rcm-1-2.dtbo     ti/k3-j721e-fpdlink-imx390-rcm-1-3.dtbo     ti/k3-j721e-sk.dtb     ti/k3-j721e-fpdlink-sk-fusion.dtbo     ti/k3-j721e-sk-csi2-ov5640.dtbo     ti/k3-j721e-sk-rpi-cam-imx219.dtbo      ti/k3-j721e-vision-apps.dtbo     ti/k3-j721e-edgeai-apps.dtbo     ti/k3-j721e-sk-rpi-hdr-ehrpwm.dtbo ; do \
            echo $DTB; cp ${LINUX_SDK}/${RELATIVE_KERNEL_PATH}/arch/arm64/boot/dts/$DTB ${PREBUILT_PATH}/kernel/ti/; \
        done
        cp ${LINUX_SDK}/${RELATIVE_PREBUILT_PATH}/fitImage.its ${PREBUILT_PATH}/kernel
        cp ${LINUX_SDK}/${RELATIVE_KERNEL_PATH}/arch/arm64/boot/Image ${PREBUILT_PATH}/kernel
    fi

    if [ ${FULL_BINARY_SIGNED} -eq 1 ]; then
        #try to bring out the staff to build app.
        make -C ${RTOS_SDK}/vision_apps vision_apps
        check_local_file_exist_then_exit ${RTOS_SDK}/vision_apps/out/J7/R5F/FREERTOS/release/vx_app_rtos_linux_mcu2_0.out
        check_local_file_exist_then_exit ${RTOS_SDK}/vision_apps/out/J7/R5F/FREERTOS/release/vx_app_rtos_linux_mcu2_1.out
        check_local_file_exist_then_exit ${RTOS_SDK}/vision_apps/out/J7/C66/FREERTOS/release/vx_app_rtos_linux_c6x_1.out
        check_local_file_exist_then_exit ${RTOS_SDK}/vision_apps/out/J7/C66/FREERTOS/release/vx_app_rtos_linux_c6x_2.out
        check_local_file_exist_then_exit ${RTOS_SDK}/vision_apps/out/J7/C71/FREERTOS/release/vx_app_rtos_linux_c7x_1.out
        ${RTOS_SDK}/${RELATIVE_ARM_LLVM_PATH}/bin/tiarmstrip -p ${RTOS_SDK}/vision_apps/out/J7/R5F/FREERTOS/release/vx_app_rtos_linux_mcu2_0.out -o ${PREBUILT_PATH}/vx_app_rtos_linux_mcu2_0.out
        ${RTOS_SDK}/${RELATIVE_ARM_LLVM_PATH}/bin/tiarmstrip -p ${RTOS_SDK}/vision_apps/out/J7/R5F/FREERTOS/release/vx_app_rtos_linux_mcu2_1.out -o ${PREBUILT_PATH}/vx_app_rtos_linux_mcu2_1.out
        ${RTOS_SDK}/${RELATIVE_CGT_C6000_PATH}/bin/strip6x -p ${RTOS_SDK}/vision_apps/out/J7/C66/FREERTOS/release/vx_app_rtos_linux_c6x_1.out -o ${PREBUILT_PATH}/vx_app_rtos_linux_c6x_1.out
        ${RTOS_SDK}/${RELATIVE_CGT_C6000_PATH}/bin/strip6x -p ${RTOS_SDK}/vision_apps/out/J7/C66/FREERTOS/release/vx_app_rtos_linux_c6x_2.out -o ${PREBUILT_PATH}/vx_app_rtos_linux_c6x_2.out
        ${RTOS_SDK}/${RELATIVE_CGT_C7x_PATH}/bin/strip7x -p ${RTOS_SDK}/vision_apps/out/J7/C71/FREERTOS/release/vx_app_rtos_linux_c7x_1.out -o ${PREBUILT_PATH}/vx_app_rtos_linux_c7x_1.out

        check_local_file_exist ${RTOS_SDK}/vision_apps/out/J7/R5F/FREERTOS/release/vx_app_rtos_linux_mcu3_0.out
        if [ $? -eq 0 ]
        then
        ${RTOS_SDK}/${RELATIVE_ARM_LLVM_PATH}/bin/tiarmstrip -p ${RTOS_SDK}/vision_apps/out/J7/R5F/FREERTOS/release/vx_app_rtos_linux_mcu3_0.out -o ${PREBUILT_PATH}/vx_app_rtos_linux_mcu3_0.out
        fi
        check_local_file_exist ${RTOS_SDK}/vision_apps/out/J7/R5F/FREERTOS/release/vx_app_rtos_linux_mcu3_1.out
        if [ $? -eq 0 ]
        then
        ${RTOS_SDK}/${RELATIVE_ARM_LLVM_PATH}/bin/tiarmstrip -p ${RTOS_SDK}/vision_apps/out/J7/R5F/FREERTOS/release/vx_app_rtos_linux_mcu3_1.out -o ${PREBUILT_PATH}/vx_app_rtos_linux_mcu3_1.out
        fi
    fi

    echo "extract this tool from LINUX SDK, done"
    echo "#######################################################################"
}
function SBL_extract_do {
    echo "#######################################################################"
    echo "try to extract this tool from RTOS/LINUX SDK"
    
    echo "firstly, make sure patch/${SDK_VERSION}/0001-enable-combined-appimage.patch has been applied to RTOS SDK"
    echo "thirdly, make sure patch/${SDK_VERSION}/0001-vision-apps-integrated.patch has been applied to LINUX SDK"
    read -p "press ENTER to continue..."



if [ ! ${FULL_BINARY_SIGNED} -eq 1 ]; then
    echo "make sure patch/${SDK_VERSION}/0001-do-NOT-do-authentication-for-other-binaries-in-SBL.patch has been applied to RTOS SDK"
    read -p "press ENTER to continue..."
fi
    cp ${RTOS_SDK}/${RELATIVE_PDK_PATH}/packages/ti/build/makerules/k3_dev_mpk.pem $cwd/${SOC}_${SDK_VERSION}

    #try to bring out the staff to build sbl.
    make -C ${RTOS_SDK}/${RELATIVE_PDK_PATH}/packages/ti/build/ BOARD=${BOARD} CORE=mcu1_0 BUILD_PROFILE=release sbl_mmcsd_img_hs
    echo "cp ${RTOS_SDK}/${RELATIVE_PDK_PATH}/packages/ti/build/makerules/x509CertificateGen.sh $cwd/${SOC}_${SDK_VERSION}"
    cp ${RTOS_SDK}/${RELATIVE_PDK_PATH}/packages/ti/build/makerules/x509CertificateGen.sh $cwd/${SOC}_${SDK_VERSION}
    cp ${RTOS_SDK}/${RELATIVE_PDK_PATH}/packages/ti/boot/sbl/binary/j721e_evm_hs/mmcsd/bin/sbl_mmcsd_img_mcu1_0_release.bin ${PREBUILT_PATH}

    #try to bring out the staff to build tifs.
    cp ${RTOS_SDK}/${RELATIVE_PDK_PATH}/packages/ti/drv/sciclient/soc/sysfw/binaries/ti-fs-firmware-${SYSFW_SOC}-hs-cert.bin ${PREBUILT_PATH}
    cp ${RTOS_SDK}/${RELATIVE_PDK_PATH}/packages/ti/drv/sciclient/soc/sysfw/binaries/ti-fs-firmware-${SYSFW_SOC}-hs-enc.bin ${PREBUILT_PATH}

    if [ ${FULL_BINARY_SIGNED} -eq 1 ]; then
        #try to bring out the staff to build mcu1_0
        make -C ${RTOS_SDK}/${RELATIVE_PDK_PATH}/packages/ti/build/ BOARD=${BOARD} CORE=mcu1_0 BUILD_PROFILE=release ipc_echo_testb_freertos
        cp ${RTOS_SDK}/${RELATIVE_PDK_PATH}/packages/ti/binary/ipc_echo_testb_freertos/bin/j721e_evm/ipc_echo_testb_freertos_mcu1_0_release_strip.xer5f ${PREBUILT_PATH}
    fi

    #try to bring out the staff to build remote cores.
    check_local_file_exist_then_exit ${RTOS_SDK}/vision_apps/out/J7/R5F/FREERTOS/release/vx_app_rtos_linux_mcu2_0.out
    check_local_file_exist_then_exit ${RTOS_SDK}/vision_apps/out/J7/R5F/FREERTOS/release/vx_app_rtos_linux_mcu2_1.out
    check_local_file_exist_then_exit ${RTOS_SDK}/vision_apps/out/J7/C66/FREERTOS/release/vx_app_rtos_linux_c6x_1.out
    check_local_file_exist_then_exit ${RTOS_SDK}/vision_apps/out/J7/C66/FREERTOS/release/vx_app_rtos_linux_c6x_2.out
    check_local_file_exist_then_exit ${RTOS_SDK}/vision_apps/out/J7/C71/FREERTOS/release/vx_app_rtos_linux_c7x_1.out
    ${RTOS_SDK}/${RELATIVE_ARM_LLVM_PATH}/bin/tiarmstrip -p ${RTOS_SDK}/vision_apps/out/J7/R5F/FREERTOS/release/vx_app_rtos_linux_mcu2_0.out -o ${PREBUILT_PATH}/vx_app_rtos_linux_mcu2_0.out
    ${RTOS_SDK}/${RELATIVE_ARM_LLVM_PATH}/bin/tiarmstrip -p ${RTOS_SDK}/vision_apps/out/J7/R5F/FREERTOS/release/vx_app_rtos_linux_mcu2_1.out -o ${PREBUILT_PATH}/vx_app_rtos_linux_mcu2_1.out
    ${RTOS_SDK}/${RELATIVE_CGT_C6000_PATH}/bin/strip6x -p ${RTOS_SDK}/vision_apps/out/J7/C66/FREERTOS/release/vx_app_rtos_linux_c6x_1.out -o ${PREBUILT_PATH}/vx_app_rtos_linux_c6x_1.out
    ${RTOS_SDK}/${RELATIVE_CGT_C6000_PATH}/bin/strip6x -p ${RTOS_SDK}/vision_apps/out/J7/C66/FREERTOS/release/vx_app_rtos_linux_c6x_2.out -o ${PREBUILT_PATH}/vx_app_rtos_linux_c6x_2.out
    ${RTOS_SDK}/${RELATIVE_CGT_C7x_PATH}/bin/strip7x -p ${RTOS_SDK}/vision_apps/out/J7/C71/FREERTOS/release/vx_app_rtos_linux_c7x_1.out -o ${PREBUILT_PATH}/vx_app_rtos_linux_c7x_1.out
    check_local_file_exist ${RTOS_SDK}/vision_apps/out/J7/R5F/FREERTOS/release/vx_app_rtos_linux_mcu3_0.out
    if [ $? -eq 0 ]
    then
    ${RTOS_SDK}/${RELATIVE_ARM_LLVM_PATH}/bin/tiarmstrip -p ${RTOS_SDK}/vision_apps/out/J7/R5F/FREERTOS/release/vx_app_rtos_linux_mcu3_0.out -o ${PREBUILT_PATH}/vx_app_rtos_linux_mcu3_0.out
    fi
    check_local_file_exist ${RTOS_SDK}/vision_apps/out/J7/R5F/FREERTOS/release/vx_app_rtos_linux_mcu3_1.out
    if [ $? -eq 0 ]
    then
    ${RTOS_SDK}/${RELATIVE_ARM_LLVM_PATH}/bin/tiarmstrip -p ${RTOS_SDK}/vision_apps/out/J7/R5F/FREERTOS/release/vx_app_rtos_linux_mcu3_1.out -o ${PREBUILT_PATH}/vx_app_rtos_linux_mcu3_1.out
    fi

    #try to bring out the staff to build app
    make -C ${LINUX_SDK} linux-dtbs
    cp ${LINUX_SDK}/${RELATIVE_KERNEL_PATH}/arch/arm64/boot/dts/ti/k3-j721e-common-proc-board.dtb ${LINUX_SDK}/board-support/prebuilt-images/base-board.dtb

    export LINUX_SDK        
    make -C ${RTOS_SDK}/${RELATIVE_PDK_PATH}/packages/ti/boot/sbl/tools/combined_appimage/
    cp ${RTOS_SDK}/${RELATIVE_PDK_PATH}/packages/ti/boot/sbl/tools/combined_appimage/bin/j721e_evm/combined_opt.appimage ${PREBUILT_PATH}


    echo "extract this tool from LINUX SDK, done"
    echo "#######################################################################"
}
#do

if [ ${EXTRACT_TOOL_FLAG} -eq 1 ]; then
    if [ ${BOOT_SPL_MODE} -eq 1 ]; then
        SPL_extract_do
    else
        SBL_extract_do
    fi
fi

function SPL_make_prepare {
    echo "prepare environment to make targets"
    read -p "please set KEY_FILE path, example:[${cwd}/${SOC}_${SDK_VERSION}/core-secdev-k3/keys/custMpk.pem]:" KEY_FILE
    if [ ! -n "$KEY_FILE" ]; then
        KEY_FILE=${cwd}/${SOC}_${SDK_VERSION}/core-secdev-k3/keys/custMpk.pem
    fi
    read -p "please select silicon type[hs/hsp], default is hs:" SOC_HS_TYPE
    if [ ! -n "$SOC_HS_TYPE" ]; then
        SOC_HS_TYPE=hs
    fi
    if [[ ${SOC_HS_TYPE} != hs && ${SOC_HS_TYPE} != hsp ]]; then
        echo "silicon type is invalid, ${SOC_HS_TYPE}, exit"
        exit
    fi

    if [ ${SOC_HS_TYPE} = hsp ]; then
        #in case of HSP, ti-fs-firmware-j721e-hs.bin need be generated in TIFS, this is the plain binary.
        read -p "please set TIFS path, example:[${cwd}/${SOC}_${SDK_VERSION}/ti-fs-firmware-j721e-hs.bin]:" TIFS_PLAIN_FILE
        if [ ! -n "$TIFS_PLAIN_FILE" ]; then
            TIFS_PLAIN_FILE=${cwd}/${SOC}_${SDK_VERSION}/ti-fs-firmware-${SYSFW_SOC}-hs.bin
        fi
        check_local_file_exist_then_exit ${TIFS_PLAIN_FILE}
        cp ${TIFS_PLAIN_FILE} ${cwd}/${SOC}_${SDK_VERSION}/ti-fs-firmware-${SYSFW_SOC}-hs.bin_${SDK_VERSION}
        #echo "currently, the tool for hs prime silicon still under development. exit"
        #exit
    fi
    check_local_file_exist_then_exit ${KEY_FILE}

    set_uid
    set_prebuilt_path
    set_output_path

    check_local_folder_exist_then_exit ${SOC}_${SDK_VERSION}/${K3_IMAGE_GEN}
    check_local_folder_exist_then_exit ${SOC}_${SDK_VERSION}/${CORE_SECDEV}

    check_local_file_exist_then_exit ${PREBUILT_PATH}/${RCORE_FLAG}_${SPL_FLAG}_u-boot-spl.bin
    check_local_file_exist_then_exit ${SOC}_${SDK_VERSION}/k3_gen_x509_cert.sh

    if [ ${SOC_HS_TYPE} = hs ]; then
        check_local_file_exist_then_exit ${PREBUILT_PATH}/ti-fs-firmware-${SYSFW_SOC}-hs-cert.bin
        check_local_file_exist_then_exit ${PREBUILT_PATH}/ti-fs-firmware-${SYSFW_SOC}-hs-enc.bin
    else
        #in case of HSP, ti-fs-firmware-j721e-hs.bin need be generated in TIFS, this is the plain binary.
        cp ${cwd}/${SOC}_${SDK_VERSION}/ti-fs-firmware-${SYSFW_SOC}-hs.bin_${SDK_VERSION} ${PREBUILT_PATH}/ti-fs-firmware-${SYSFW_SOC}-hs.bin
        check_local_file_exist_then_exit ${PREBUILT_PATH}/ti-fs-firmware-${SYSFW_SOC}-hs.bin
        #echo "currently, the tool for hs prime silicon still under development. exit"
        #EXIT
    fi

    check_local_file_exist_then_exit ${PREBUILT_PATH}/pm-cfg.bin
    check_local_file_exist_then_exit ${PREBUILT_PATH}/sec-cfg.bin
    check_local_file_exist_then_exit ${PREBUILT_PATH}/board-cfg.bin
    check_local_file_exist_then_exit ${PREBUILT_PATH}/rm-cfg.bin


    if [ ${FULL_BINARY_SIGNED} -eq 1 ]; then
        check_local_file_exist_then_exit ${PREBUILT_PATH}/${ACORE_FLAG}_${SPL_FLAG}_k3-j721e-common-proc-board.dtb
        check_local_file_exist_then_exit ${PREBUILT_PATH}/${ACORE_FLAG}_${SPL_FLAG}_k3-j721e-tps65917-proc-board.dtb
        check_local_file_exist_then_exit ${PREBUILT_PATH}/${ACORE_FLAG}_${SPL_FLAG}_k3-j721e-sk.dtb
        check_local_file_exist_then_exit ${PREBUILT_PATH}/${ACORE_FLAG}_${SPL_FLAG}_u-boot-spl-nodtb.bin
        check_local_file_exist_then_exit ${PREBUILT_PATH}/bl31.bin
        check_local_file_exist_then_exit ${PREBUILT_PATH}/bl32.bin
        check_local_file_exist_then_exit ${PREBUILT_PATH}/ipc_echo_testb_mcu1_0_release_strip.xer5f
        check_local_file_exist_then_exit ${SOC}_${SDK_VERSION}/k3_fit_atf.sh
    fi

    
    if [ ${FULL_BINARY_SIGNED} -eq 1 ]; then
        check_local_file_exist_then_exit ${PREBUILT_PATH}/${ACORE_FLAG}_k3-j721e-common-proc-board.dtb
        check_local_file_exist_then_exit ${PREBUILT_PATH}/${ACORE_FLAG}_k3-j721e-tps65917-proc-board.dtb
        check_local_file_exist_then_exit ${PREBUILT_PATH}/${ACORE_FLAG}_k3-j721e-sk.dtb
        check_local_file_exist_then_exit ${PREBUILT_PATH}/${ACORE_FLAG}_u-boot-nodtb.bin
    fi


    if [ ${FULL_BINARY_SIGNED} -eq 1 ]; then
        check_local_file_exist_then_exit ${PREBUILT_PATH}/kernel/Image
    fi

    if [ ${FULL_BINARY_SIGNED} -eq 1 ]; then
        check_local_file_exist_then_exit ${PREBUILT_PATH}/vx_app_rtos_linux_mcu2_0.out
        check_local_file_exist_then_exit ${PREBUILT_PATH}/vx_app_rtos_linux_mcu2_1.out
        check_local_file_exist_then_exit ${PREBUILT_PATH}/vx_app_rtos_linux_c6x_1.out
        check_local_file_exist_then_exit ${PREBUILT_PATH}/vx_app_rtos_linux_c6x_2.out
        check_local_file_exist_then_exit ${PREBUILT_PATH}/vx_app_rtos_linux_c7x_1.out

        check_local_file_exist ${PREBUILT_PATH}/vx_app_rtos_linux_mcu3_0.out
        check_local_file_exist ${PREBUILT_PATH}/vx_app_rtos_linux_mcu3_1.out
    fi

    print_debug_info
}

function SBL_make_prepare {
    echo "prepare environment to make targets"
    read -p "please set KEY_FILE path, example:[${cwd}/${SOC}_${SDK_VERSION}/k3_dev_mpk.pem]:" KEY_FILE
    if [ ! -n "$KEY_FILE" ]; then
        KEY_FILE=${cwd}/${SOC}_${SDK_VERSION}/k3_dev_mpk.pem
    fi
    read -p "please select silicon type[hs/hsp], default is hs:" SOC_HS_TYPE
    if [ ! -n "$SOC_HS_TYPE" ]; then
        SOC_HS_TYPE=hs
    fi
    if [[ ${SOC_HS_TYPE} != hs && ${SOC_HS_TYPE} != hsp ]]; then
        echo "silicon type is invalid, ${SOC_HS_TYPE}, exit"
        exit
    fi

    if [ ${SOC_HS_TYPE} = hsp ]; then
        #in case of HSP, ti-fs-firmware-j721e-hs.bin need be generated in TIFS, this is the plain binary.
        read -p "please set TIFS path, example:[${cwd}/${SOC}_${SDK_VERSION}/ti-fs-firmware-j721e-hs.bin]:" TIFS_PLAIN_FILE
        if [ ! -n "$TIFS_PLAIN_FILE" ]; then
            TIFS_PLAIN_FILE=${cwd}/${SOC}_${SDK_VERSION}/ti-fs-firmware-${SYSFW_SOC}-hs.bin
        fi
        check_local_file_exist_then_exit ${TIFS_PLAIN_FILE}
        cp ${TIFS_PLAIN_FILE} ${cwd}/${SOC}_${SDK_VERSION}/ti-fs-firmware-${SYSFW_SOC}-hs.bin_${SDK_VERSION}
        #echo "currently, the tool for hs prime silicon still under development. exit"
        #exit
    fi
    check_local_file_exist_then_exit ${KEY_FILE}

    set_uid
    set_prebuilt_path
    set_output_path

    check_local_file_exist_then_exit ${SOC}_${SDK_VERSION}/x509CertificateGen.sh
    check_local_file_exist_then_exit ${PREBUILT_PATH}/sbl_mmcsd_img_mcu1_0_release.bin

    if [ ${SOC_HS_TYPE} = hs ]; then
        check_local_file_exist_then_exit ${PREBUILT_PATH}/ti-fs-firmware-${SYSFW_SOC}-hs-cert.bin
        check_local_file_exist_then_exit ${PREBUILT_PATH}/ti-fs-firmware-${SYSFW_SOC}-hs-enc.bin
    else
        #in case of HSP, ti-fs-firmware-j721e-hs.bin need be generated in TIFS, this is the plain binary.
        cp ${cwd}/${SOC}_${SDK_VERSION}/ti-fs-firmware-${SYSFW_SOC}-hs.bin_${SDK_VERSION} ${PREBUILT_PATH}/ti-fs-firmware-${SYSFW_SOC}-hs.bin
        check_local_file_exist_then_exit ${PREBUILT_PATH}/ti-fs-firmware-${SYSFW_SOC}-hs.bin
        #echo "currently, the tool for hs prime silicon still under development. exit"
        #EXIT
    fi
    check_local_file_exist_then_exit ${PREBUILT_PATH}/combined_opt.appimage

    check_local_file_exist_then_exit ${PREBUILT_PATH}/vx_app_rtos_linux_mcu2_0.out
    check_local_file_exist_then_exit ${PREBUILT_PATH}/vx_app_rtos_linux_mcu2_1.out
    check_local_file_exist_then_exit ${PREBUILT_PATH}/vx_app_rtos_linux_c6x_1.out
    check_local_file_exist_then_exit ${PREBUILT_PATH}/vx_app_rtos_linux_c6x_2.out
    check_local_file_exist_then_exit ${PREBUILT_PATH}/vx_app_rtos_linux_c7x_1.out
    check_local_file_exist ${PREBUILT_PATH}/vx_app_rtos_linux_mcu3_0.out
    check_local_file_exist ${PREBUILT_PATH}/vx_app_rtos_linux_mcu3_1.out

    print_debug_info
}

#prepare environment
if [ ${MAKE_TARGET_FLAG} -eq 1 ]; then
    if [ ${BOOT_SPL_MODE} -eq 1 ]; then
        SPL_make_prepare
    else
        SBL_make_prepare
    fi
fi

function SPL_make_do {
    echo "#######################################################################"
    echo "try to make targets"

    echo "sign tiboot3.bin..."
    ./${SOC}_${SDK_VERSION}/k3_gen_x509_cert.sh -c 16 -b ${PREBUILT_PATH}/${RCORE_FLAG}_${SPL_FLAG}_u-boot-spl.bin \
		-o ${OUTPUT_PATH}/tiboot3.bin -l 0x41c00000 -r 1 -k ${KEY_FILE} -d 4


    if [ ${SOC_HS_TYPE} = hs ]; then
        echo "sign sysfw.itb..."
        #replace UID in x509 certifcate
        sed -e "s/0000000000000000000000000000000000000000000000000000000000000000/${SOC_UID}/g" ${SOC}_${SDK_VERSION}/${K3_IMAGE_GEN}/gen_x509_cert.sh > ${SOC}_${SDK_VERSION}/${K3_IMAGE_GEN}/tmp_gen_x509_cert.sh
        sudo chmod +x ${SOC}_${SDK_VERSION}/${K3_IMAGE_GEN}/tmp_gen_x509_cert.sh
        ${SOC}_${SDK_VERSION}/${K3_IMAGE_GEN}/tmp_gen_x509_cert.sh -d -c m3 -b ${PREBUILT_PATH}/ti-fs-firmware-j721e-hs-cert.bin -o ${PREBUILT_PATH}/ti-fs-firmware-j721e-hs-certs.bin -l 0x40000 -k ${KEY_FILE} -r 1
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/pm-cfg.bin ${PREBUILT_PATH}/pm-cfg.bin-signed
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/sec-cfg.bin ${PREBUILT_PATH}/sec-cfg.bin-signed
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/board-cfg.bin ${PREBUILT_PATH}/board-cfg.bin-signed
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/rm-cfg.bin ${PREBUILT_PATH}/rm-cfg.bin-signed
        cd ${SOC}_${SDK_VERSION}/${K3_IMAGE_GEN}
        ./gen_its.sh ${SYSFW_SOC} hs evm ${PREBUILT_PATH}/sysfw.bin-hs ${PREBUILT_PATH}/board-cfg.bin-signed ${PREBUILT_PATH}/pm-cfg.bin-signed ${PREBUILT_PATH}/rm-cfg.bin-signed ${PREBUILT_PATH}/sec-cfg.bin-signed > ${PREBUILT_PATH}/sysfw-j721e-evm.its
        cd ${cwd}
        cat ${PREBUILT_PATH}/ti-fs-firmware-${SYSFW_SOC}-hs-certs.bin ${PREBUILT_PATH}/ti-fs-firmware-${SYSFW_SOC}-hs-enc.bin > ${PREBUILT_PATH}/sysfw.bin-hs
        ${PREBUILT_PATH}/${RCORE_FLAG}_${SPL_FLAG}_mkimage -f ${PREBUILT_PATH}/sysfw-j721e-evm.its -r ${OUTPUT_PATH}/sysfw.itb
    else
        echo "sign sysfw.itb..."
        #replace UID in x509 certifcate
        sed -e "s/0000000000000000000000000000000000000000000000000000000000000000/${SOC_UID}/g" ${SOC}_${SDK_VERSION}/${K3_IMAGE_GEN}/gen_x509_cert.sh > ${SOC}_${SDK_VERSION}/${K3_IMAGE_GEN}/tmp_gen_x509_cert.sh
        sudo chmod +x ${SOC}_${SDK_VERSION}/${K3_IMAGE_GEN}/tmp_gen_x509_cert.sh
        ${SOC}_${SDK_VERSION}/${K3_IMAGE_GEN}/tmp_gen_x509_cert.sh -c m3 -b ${PREBUILT_PATH}/ti-fs-firmware-j721e-hs.bin -o ${PREBUILT_PATH}/sysfw.bin-hs -l 0x40000 -k ${KEY_FILE} -r 1
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/pm-cfg.bin ${PREBUILT_PATH}/pm-cfg.bin-signed
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/sec-cfg.bin ${PREBUILT_PATH}/sec-cfg.bin-signed
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/board-cfg.bin ${PREBUILT_PATH}/board-cfg.bin-signed
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/rm-cfg.bin ${PREBUILT_PATH}/rm-cfg.bin-signed
        cd ${SOC}_${SDK_VERSION}/${K3_IMAGE_GEN}
        ./gen_its.sh ${SYSFW_SOC} hs evm ${PREBUILT_PATH}/sysfw.bin-hs ${PREBUILT_PATH}/board-cfg.bin-signed ${PREBUILT_PATH}/pm-cfg.bin-signed ${PREBUILT_PATH}/rm-cfg.bin-signed ${PREBUILT_PATH}/sec-cfg.bin-signed > ${PREBUILT_PATH}/sysfw-j721e-evm.its
        cd ${cwd}
        ${PREBUILT_PATH}/${RCORE_FLAG}_${SPL_FLAG}_mkimage -f ${PREBUILT_PATH}/sysfw-j721e-evm.its -r ${OUTPUT_PATH}/sysfw.itb
        #need development.
    fi


    if [ ${FULL_BINARY_SIGNED} -eq 1 ]; then
        echo "sign tispl.bin..."
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/${ACORE_FLAG}_${SPL_FLAG}_k3-j721e-common-proc-board.dtb ${PREBUILT_PATH}/k3-j721e-common-proc-board.dtb_HS
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/${ACORE_FLAG}_${SPL_FLAG}_k3-j721e-tps65917-proc-board.dtb ${PREBUILT_PATH}/k3-j721e-tps65917-proc-board.dtb_HS
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/${ACORE_FLAG}_${SPL_FLAG}_k3-j721e-sk.dtb ${PREBUILT_PATH}/k3-j721e-sk.dtb_HS
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/${ACORE_FLAG}_${SPL_FLAG}_u-boot-${SPL_FLAG}-nodtb.bin ${PREBUILT_PATH}/u-boot-${SPL_FLAG}-nodtb.bin_HS

        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/bl31.bin ${PREBUILT_PATH}/bl31.bin.signed
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/bl32.bin ${PREBUILT_PATH}/bl32.bin.signed
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/ipc_echo_testb_mcu1_0_release_strip.xer5f ${PREBUILT_PATH}/ipc_echo_testb_mcu1_0_release_strip.xer5f.signed
        
        export ATF=${PREBUILT_PATH}/bl31.bin.signed
        export TEE=${PREBUILT_PATH}/bl32.bin.signed
        export DM=${PREBUILT_PATH}/ipc_echo_testb_mcu1_0_release_strip.xer5f.signed
        export IS_HS=1
        export BIN_PATH=${PREBUILT_PATH}/u-boot-${SPL_FLAG}-nodtb.bin
        cp ${SOC}_${SDK_VERSION}/k3_fit_atf.sh ${SOC}_${SDK_VERSION}/tmp_k3_fit_atf.sh
        sed -e 's/spl\/u-boot-spl-nodtb.bin/${temp}/g' ${SOC}_${SDK_VERSION}/tmp_k3_fit_atf.sh > ${SOC}_${SDK_VERSION}/tmp1_k3_fit_atf.sh
        sed -e 9a\temp=${BIN_PATH} ${SOC}_${SDK_VERSION}/tmp1_k3_fit_atf.sh > ${SOC}_${SDK_VERSION}/k3_fit_atf.sh
        ./${SOC}_${SDK_VERSION}/k3_fit_atf.sh 0x70000000 ${PREBUILT_PATH}/k3-j721e-common-proc-board.dtb ${PREBUILT_PATH}/k3-j721e-tps65917-proc-board.dtb ${PREBUILT_PATH}/k3-j721e-sk.dtb > ${PREBUILT_PATH}/u-boot-spl-k3_HS.its    
        ${PREBUILT_PATH}/${ACORE_FLAG}_mkimage -f ${PREBUILT_PATH}/u-boot-spl-k3_HS.its -E ${OUTPUT_PATH}/tispl.bin

        #backup the dtbs...
        mv ${PREBUILT_PATH}/k3-j721e-common-proc-board.dtb_HS ${PREBUILT_PATH}/${ACORE_FLAG}_${SPL_FLAG}_k3-j721e-common-proc-board.dtb_HS
        mv ${PREBUILT_PATH}/k3-j721e-tps65917-proc-board.dtb_HS ${PREBUILT_PATH}/${ACORE_FLAG}_${SPL_FLAG}_k3-j721e-tps65917-proc-board.dtb_HS
        mv ${PREBUILT_PATH}/k3-j721e-sk.dtb_HS ${PREBUILT_PATH}/${ACORE_FLAG}_${SPL_FLAG}_k3-j721e-sk.dtb_HS
        mv ${PREBUILT_PATH}/u-boot-${SPL_FLAG}-nodtb.bin_HS ${PREBUILT_PATH}/${ACORE_FLAG}_u-boot-${SPL_FLAG}-nodtb.bin_HS
    fi

    if [ ${FULL_BINARY_SIGNED} -eq 1 ]; then
        echo "sign u-boot.img..."
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/${ACORE_FLAG}_k3-j721e-common-proc-board.dtb ${PREBUILT_PATH}/k3-j721e-common-proc-board.dtb_HS
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/${ACORE_FLAG}_k3-j721e-tps65917-proc-board.dtb ${PREBUILT_PATH}/k3-j721e-tps65917-proc-board.dtb_HS
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/${ACORE_FLAG}_k3-j721e-sk.dtb ${PREBUILT_PATH}/k3-j721e-sk.dtb_HS
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/${ACORE_FLAG}_u-boot-nodtb.bin ${PREBUILT_PATH}/u-boot-nodtb.bin_HS

        cat ${PREBUILT_PATH}/temp.txt > ${temp_release}
        echo "${temp_release}"

        ${PREBUILT_PATH}/${ACORE_FLAG}_mkimage -f auto -A arm -T firmware -C none -O u-boot -a 0x80800000 -e 0x80800000 \
        -n '${temp_release}' -E -b ${PREBUILT_PATH}/k3-j721e-common-proc-board.dtb_HS \
        -b ${PREBUILT_PATH}/k3-j721e-tps65917-proc-board.dtb_HS -b ${PREBUILT_PATH}/k3-j721e-sk.dtb_HS \
        -d ${PREBUILT_PATH}/u-boot-nodtb.bin_HS ${OUTPUT_PATH}/u-boot.img
    fi

    if [ ${FULL_BINARY_SIGNED} -eq 1 ]; then
        echo "sign kernel..."
        for DTB in      ti/k3-j721e-common-proc-board.dtb     ti/k3-j721e-proc-board-tps65917.dtb     ti/k3-j721e-common-proc-board-infotainment.dtbo     ti/k3-j721e-cpb-csi2-ov5640.dtbo     ti/k3-j721e-fpdlink-cpb-fusion.dtbo     ti/k3-j721e-fpdlink-imx390-cm-0-0.dtbo     ti/k3-j721e-fpdlink-imx390-cm-0-1.dtbo     ti/k3-j721e-fpdlink-imx390-cm-0-2.dtbo     ti/k3-j721e-fpdlink-imx390-cm-0-3.dtbo     ti/k3-j721e-fpdlink-imx390-cm-1-0.dtbo     ti/k3-j721e-fpdlink-imx390-cm-1-1.dtbo     ti/k3-j721e-fpdlink-imx390-cm-1-2.dtbo     ti/k3-j721e-fpdlink-imx390-cm-1-3.dtbo     ti/k3-j721e-fpdlink-imx390-rcm-0-0.dtbo     ti/k3-j721e-fpdlink-imx390-rcm-0-1.dtbo     ti/k3-j721e-fpdlink-imx390-rcm-0-2.dtbo     ti/k3-j721e-fpdlink-imx390-rcm-0-3.dtbo     ti/k3-j721e-fpdlink-imx390-rcm-1-0.dtbo     ti/k3-j721e-fpdlink-imx390-rcm-1-1.dtbo     ti/k3-j721e-fpdlink-imx390-rcm-1-2.dtbo     ti/k3-j721e-fpdlink-imx390-rcm-1-3.dtbo     ti/k3-j721e-sk.dtb     ti/k3-j721e-fpdlink-sk-fusion.dtbo     ti/k3-j721e-sk-csi2-ov5640.dtbo     ti/k3-j721e-sk-rpi-cam-imx219.dtbo      ti/k3-j721e-vision-apps.dtbo     ti/k3-j721e-edgeai-apps.dtbo     ti/k3-j721e-sk-rpi-hdr-ehrpwm.dtbo ; do \
            ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/kernel/${DTB} ${PREBUILT_PATH}/kernel/${DTB}.sec; \
        done
        cp -rf ${PREBUILT_PATH}/kernel/ti/*.sec ${PREBUILT_PATH}/kernel
        rm -rf ${PREBUILT_PATH}/kernel/ti/*.sec

        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/kernel/Image ${PREBUILT_PATH}/kernel/Image.sec
        
        cd ${PREBUILT_PATH}/kernel
        mkimage -f fitImage.its -r ${OUTPUT_PATH}/fitImage
        cd ${cwd}
    fi

    if [ ${FULL_BINARY_SIGNED} -eq 1 ]; then
        echo "sign app..."
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/vx_app_rtos_linux_mcu2_0.out ${OUTPUT_PATH}/vx_app_rtos_linux_mcu2_0.out.signed
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/vx_app_rtos_linux_mcu2_1.out ${OUTPUT_PATH}/vx_app_rtos_linux_mcu2_1.out.signed
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/vx_app_rtos_linux_c6x_1.out ${OUTPUT_PATH}/vx_app_rtos_linux_c6x_1.out.signed
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/vx_app_rtos_linux_c6x_2.out ${OUTPUT_PATH}/vx_app_rtos_linux_c6x_2.out.signed
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/vx_app_rtos_linux_c7x_1.out ${OUTPUT_PATH}/vx_app_rtos_linux_c7x_1.out.signed
        
        cp ${PREBUILT_PATH}/vx_app_rtos_linux_mcu2_0.out ${OUTPUT_PATH}
        cp ${PREBUILT_PATH}/vx_app_rtos_linux_mcu2_1.out ${OUTPUT_PATH}
        cp ${PREBUILT_PATH}/vx_app_rtos_linux_c6x_1.out ${OUTPUT_PATH}
        cp ${PREBUILT_PATH}/vx_app_rtos_linux_c6x_2.out ${OUTPUT_PATH}
        cp ${PREBUILT_PATH}/vx_app_rtos_linux_c7x_1.out ${OUTPUT_PATH}
        cp ${PREBUILT_PATH}/vx_app_rtos_linux_c7x_1.out ${OUTPUT_PATH}

        check_local_file_exist ${PREBUILT_PATH}/vx_app_rtos_linux_mcu3_0.out
        if [ $? -eq 0 ]
        then
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/vx_app_rtos_linux_mcu3_0.out ${OUTPUT_PATH}/vx_app_rtos_linux_mcu3_0.out.signed
        cp ${PREBUILT_PATH}/vx_app_rtos_linux_mcu3_0.out ${OUTPUT_PATH}
        fi
        check_local_file_exist ${PREBUILT_PATH}/vx_app_rtos_linux_mcu3_1.out
        if [ $? -eq 0 ]
        then
        ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/vx_app_rtos_linux_mcu3_1.out ${OUTPUT_PATH}/vx_app_rtos_linux_mcu3_1.out.signed
        cp ${PREBUILT_PATH}/vx_app_rtos_linux_mcu3_1.out ${OUTPUT_PATH}
        fi
    fi

    echo "sign override"
    sed -e "s/0000000000000000000000000000000000000000000000000000000000000000/${SOC_UID}/g" x509_sec_override.cfg > ${PREBUILT_PATH}/tmp_x509_sec_override.cfg
    openssl req -new -x509 -key ${KEY_FILE} -nodes -outform DER -out ${OUTPUT_PATH}/override.bin -config ${PREBUILT_PATH}/tmp_x509_sec_override.cfg -sha512
    # cp override.bin ${OUTPUT_PATH}

    echo "make targets, done"
    echo "#######################################################################"
}

function SBL_make_do {
    echo "#######################################################################"
    echo "try to make targets"

    #sign tiboot3.bin.
    /bin/chmod a+x ${SOC}_${SDK_VERSION}/x509CertificateGen.sh
    echo "./${SOC}_${SDK_VERSION}/x509CertificateGen.sh -b ${PREBUILT_PATH}/sbl_mmcsd_img_mcu1_0_release.bin -o ${OUTPUT_PATH}/tiboot3.bin -c R5 -l 0x41C00100 -k ${KEY_FILE} -d DEBUG -j DBG_FULL_ENABLE -m SPLIT_MODE"
    ./${SOC}_${SDK_VERSION}/x509CertificateGen.sh -b ${PREBUILT_PATH}/sbl_mmcsd_img_mcu1_0_release.bin \
	-o ${OUTPUT_PATH}/tiboot3.bin -c R5 -l 0x41C00100 -k ${KEY_FILE} -d DEBUG -j DBG_FULL_ENABLE -m SPLIT_MODE

    if [ ${SOC_HS_TYPE} = hs ]; then
        echo "sign tifs.bin..."
        #replace UID in x509 certifcate
        sed -e "s/0000000000000000000000000000000000000000000000000000000000000000/${SOC_UID}/g" ${SOC}_${SDK_VERSION}/x509CertificateGen.sh > ${SOC}_${SDK_VERSION}/tmp_x509CertificateGen.sh
        sudo /bin/chmod +x ${SOC}_${SDK_VERSION}/tmp_x509CertificateGen.sh
        ${SOC}_${SDK_VERSION}/tmp_x509CertificateGen.sh -b ${PREBUILT_PATH}/ti-fs-firmware-j721e-hs-cert.bin -o ${PREBUILT_PATH}/tifs_cert.bin -c DMSC_O -l 0x40000 -k ${KEY_FILE}
        /bin/cat ${PREBUILT_PATH}/tifs_cert.bin ${PREBUILT_PATH}/ti-fs-firmware-j721e-hs-enc.bin > ${OUTPUT_PATH}/tifs.bin
    else
        echo "sign tifs.bin..."
        sed -e "s/0000000000000000000000000000000000000000000000000000000000000000/${SOC_UID}/g" ${SOC}_${SDK_VERSION}/x509CertificateGen.sh > ${SOC}_${SDK_VERSION}/tmp_x509CertificateGen.sh
        sudo /bin/chmod +x ${SOC}_${SDK_VERSION}/tmp_x509CertificateGen.sh
        echo "${SOC}_${SDK_VERSION}/tmp_x509CertificateGen.sh -b ${PREBUILT_PATH}/ti-fs-firmware-j721e-hs.bin -o ${OUTPUT_PATH}/tifs.bin -c DMSC_I -l 0x40000 -k ${KEY_FILE}"
        ${SOC}_${SDK_VERSION}/tmp_x509CertificateGen.sh -b ${PREBUILT_PATH}/ti-fs-firmware-j721e-hs.bin -o ${OUTPUT_PATH}/tifs.bin -c DMSC_I -l 0x40000 -k ${KEY_FILE}
        #need development.
    fi    

    if [ ${FULL_BINARY_SIGNED} -eq 1 ]; then
        echo "sign app..."
        ./${SOC}_${SDK_VERSION}/x509CertificateGen.sh -b ${PREBUILT_PATH}/combined_opt.appimage \
	    -o ${PREBUILT_PATH}/combined_opt.appimage.signed -c R5 -l 0x41c00000 -k ${KEY_FILE}
        cp ${PREBUILT_PATH}/combined_opt.appimage.signed ${OUTPUT_PATH}/app
    else
        cp ${PREBUILT_PATH}/combined_opt.appimage ${OUTPUT_PATH}/app
    fi

    cp ${PREBUILT_PATH}/vx_app_rtos_linux_mcu2_0.out ${OUTPUT_PATH}
    cp ${PREBUILT_PATH}/vx_app_rtos_linux_mcu2_1.out ${OUTPUT_PATH}
    cp ${PREBUILT_PATH}/vx_app_rtos_linux_c6x_1.out ${OUTPUT_PATH}
    cp ${PREBUILT_PATH}/vx_app_rtos_linux_c6x_2.out ${OUTPUT_PATH}
    cp ${PREBUILT_PATH}/vx_app_rtos_linux_c7x_1.out ${OUTPUT_PATH}
    cp ${PREBUILT_PATH}/vx_app_rtos_linux_c7x_1.out ${OUTPUT_PATH}
    check_local_file_exist ${PREBUILT_PATH}/vx_app_rtos_linux_mcu3_0.out
    if [ $? -eq 0 ]
    then
    ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/vx_app_rtos_linux_mcu3_0.out ${OUTPUT_PATH}/vx_app_rtos_linux_mcu3_0.out.signed
    cp ${PREBUILT_PATH}/vx_app_rtos_linux_mcu3_0.out ${OUTPUT_PATH}
    fi
    check_local_file_exist ${PREBUILT_PATH}/vx_app_rtos_linux_mcu3_1.out
    if [ $? -eq 0 ]
    then
    ${RELATIVE_SECURE_BINARY_IMAGE_PATH} ${PREBUILT_PATH}/vx_app_rtos_linux_mcu3_1.out ${OUTPUT_PATH}/vx_app_rtos_linux_mcu3_1.out.signed
    cp ${PREBUILT_PATH}/vx_app_rtos_linux_mcu3_1.out ${OUTPUT_PATH}
    fi

    echo "sign override"
    sed -e "s/0000000000000000000000000000000000000000000000000000000000000000/${SOC_UID}/g" x509_sec_override.cfg > ${PREBUILT_PATH}/tmp_x509_sec_override.cfg
    openssl req -new -x509 -key ${KEY_FILE} -nodes -outform DER -out ${OUTPUT_PATH}/override.bin -config ${PREBUILT_PATH}/tmp_x509_sec_override.cfg -sha512
    # cp override.bin ${OUTPUT_PATH}

    echo "make targets, done"
    echo "#######################################################################"
}


#do
if [ ${MAKE_TARGET_FLAG} -eq 1 ]; then
    if [ ${BOOT_SPL_MODE} -eq 1 ]; then
        SPL_make_do
    else
        SBL_make_do
    fi
fi


function SPL_migrate_prepare {
    echo "prepare environment to migrate to SD"
    set_output_path

    check_local_file_exist_then_exit ${OUTPUT_PATH}/tiboot3.bin
    check_local_file_exist_then_exit ${OUTPUT_PATH}/sysfw.itb

    if [ ${FULL_BINARY_SIGNED} -eq 1 ]; then
        check_local_file_exist_then_exit ${OUTPUT_PATH}/tispl.bin
        check_local_file_exist_then_exit ${OUTPUT_PATH}/u-boot.img
        check_local_file_exist_then_exit ${OUTPUT_PATH}/fitImage
        check_local_file_exist_then_exit ${OUTPUT_PATH}/vx_app_rtos_linux_c6x_1.out
        check_local_file_exist_then_exit ${OUTPUT_PATH}/vx_app_rtos_linux_c6x_1.out.signed
        check_local_file_exist_then_exit ${OUTPUT_PATH}/vx_app_rtos_linux_c6x_2.out
        check_local_file_exist_then_exit ${OUTPUT_PATH}/vx_app_rtos_linux_c6x_2.out.signed
        check_local_file_exist_then_exit ${OUTPUT_PATH}/vx_app_rtos_linux_c7x_1.out
        check_local_file_exist_then_exit ${OUTPUT_PATH}/vx_app_rtos_linux_c7x_1.out.signed
        check_local_file_exist_then_exit ${OUTPUT_PATH}/vx_app_rtos_linux_mcu2_0.out
        check_local_file_exist_then_exit ${OUTPUT_PATH}/vx_app_rtos_linux_mcu2_0.out.signed
        check_local_file_exist_then_exit ${OUTPUT_PATH}/vx_app_rtos_linux_mcu2_1.out
        check_local_file_exist_then_exit ${OUTPUT_PATH}/vx_app_rtos_linux_mcu2_1.out.signed
        check_local_file_exist ${OUTPUT_PATH}/vx_app_rtos_linux_mcu3_0.out
        check_local_file_exist ${OUTPUT_PATH}/vx_app_rtos_linux_mcu3_0.out.signed
        check_local_file_exist ${OUTPUT_PATH}/vx_app_rtos_linux_mcu3_1.out
        check_local_file_exist ${OUTPUT_PATH}/vx_app_rtos_linux_mcu3_1.out.signed
    fi

    print_debug_info
}
function SBL_migrate_prepare {
    echo "prepare environment to migrate to SD"
    set_output_path

    check_local_file_exist_then_exit ${OUTPUT_PATH}/tiboot3.bin
    check_local_file_exist_then_exit ${OUTPUT_PATH}/tifs.bin
    check_local_file_exist_then_exit ${OUTPUT_PATH}/app
    
    check_local_file_exist_then_exit ${OUTPUT_PATH}/vx_app_rtos_linux_c6x_1.out
    check_local_file_exist_then_exit ${OUTPUT_PATH}/vx_app_rtos_linux_c6x_2.out
    check_local_file_exist_then_exit ${OUTPUT_PATH}/vx_app_rtos_linux_c7x_1.out
    check_local_file_exist_then_exit ${OUTPUT_PATH}/vx_app_rtos_linux_mcu2_0.out
    check_local_file_exist_then_exit ${OUTPUT_PATH}/vx_app_rtos_linux_mcu2_1.out
    check_local_file_exist ${OUTPUT_PATH}/vx_app_rtos_linux_mcu3_0.out
    check_local_file_exist ${OUTPUT_PATH}/vx_app_rtos_linux_mcu3_1.out

    print_debug_info
}


#prepare environment
if [ ${MIGRATE_TO_SD_FLAG} -eq 1 ]; then
    if [ ${BOOT_SPL_MODE} -eq 1 ]; then
        SPL_migrate_prepare
    else
        SBL_migrate_prepare
    fi
fi


function SPL_migrate_do {
    echo "#######################################################################"
    echo "try to migrate to SD"
    echo "${USER}"
    cp ${OUTPUT_PATH}/tiboot3.bin /media/${USER}/BOOT/tiboot3.bin
    cp ${OUTPUT_PATH}/sysfw.itb /media/${USER}/BOOT/sysfw.itb


    if [ ${FULL_BINARY_SIGNED} -eq 1 ]; then
        cp ${OUTPUT_PATH}/tispl.bin /media/${USER}/BOOT/tispl.bin
        cp ${OUTPUT_PATH}/u-boot.img /media/${USER}/BOOT/u-boot.img
        sudo cp ${OUTPUT_PATH}/fitImage /media/${USER}/rootfs/boot/fitImage

        rm -f /media/${USER}/rootfs/lib/firmware/j7-*-fw-sec
        rm -f /media/${USER}/rootfs/lib/firmware/vision_apps_evm/*
        rm -f /media/${USER}/rootfs/lib/firmware/vision_apps_eaik/*

        cp -r ${OUTPUT_PATH}/vx_app_rtos_linux_* /media/${USER}/rootfs/lib/firmware/vision_apps_evm/.


        ln -sr /media/${USER}/rootfs/lib/firmware/vision_apps_evm/vx_app_rtos_linux_c7x_1.out.signed /media/${USER}/rootfs/lib/firmware/j7-c71_0-fw-sec
        ln -sr /media/${USER}/rootfs/lib/firmware/vision_apps_evm/vx_app_rtos_linux_c6x_2.out.signed /media/${USER}/rootfs/lib/firmware/j7-c66_1-fw-sec
        ln -sr /media/${USER}/rootfs/lib/firmware/vision_apps_evm/vx_app_rtos_linux_c6x_1.out.signed /media/${USER}/rootfs/lib/firmware/j7-c66_0-fw-sec
        ln -sr /media/${USER}/rootfs/lib/firmware/vision_apps_evm/vx_app_rtos_linux_mcu2_1.out.signed /media/${USER}/rootfs/lib/firmware/j7-main-r5f0_1-fw-sec
        ln -sr /media/${USER}/rootfs/lib/firmware/vision_apps_evm/vx_app_rtos_linux_mcu2_0.out.signed /media/${USER}/rootfs/lib/firmware/j7-main-r5f0_0-fw-sec
        
        check_local_file_exist ${PREBUILT_PATH}/vx_app_rtos_linux_mcu3_0.out
        if [ $? -eq 0 ]
        then
        ln -sr /media/${USER}/rootfs/lib/firmware/vision_apps_evm/vx_app_rtos_linux_mcu3_0.out.signed /media/${USER}/rootfs/lib/firmware/j7-main-r5f1_0-fw-sec
        fi
        check_local_file_exist ${PREBUILT_PATH}/vx_app_rtos_linux_mcu3_1.out
        if [ $? -eq 0 ]
        then
        ln -sr /media/${USER}/rootfs/lib/firmware/vision_apps_evm/vx_app_rtos_linux_mcu3_1.out.signed /media/${USER}/rootfs/lib/firmware/j7-main-r5f1_1-fw-sec
        fi
    fi

    echo "migrate to SD, done"
    echo "#######################################################################"
}


function SBL_migrate_do {
    echo "#######################################################################"
    echo "try to migrate to SD"
    echo "${USER}"
    cp ${OUTPUT_PATH}/tiboot3.bin /media/${USER}/BOOT/tiboot3.bin
    cp ${OUTPUT_PATH}/tifs.bin /media/${USER}/BOOT/tifs.bin
    cp ${OUTPUT_PATH}/app /media/${USER}/BOOT/app


    rm -f /media/${USER}/rootfs/lib/firmware/j7-*-fw-sec
    rm -f /media/${USER}/rootfs/lib/firmware/vision_apps_evm/*
    rm -f /media/${USER}/rootfs/lib/firmware/vision_apps_eaik/*
    cp -r ${OUTPUT_PATH}/vx_app_rtos_linux_* /media/${USER}/rootfs/lib/firmware/vision_apps_evm/.
    echo "migrate to SD, done"
    echo "#######################################################################"
}

#do
if [ ${MIGRATE_TO_SD_FLAG} -eq 1 ]; then
    if [ ${BOOT_SPL_MODE} -eq 1 ]; then
        SPL_migrate_do
    else
        SBL_migrate_do
    fi
fi

exit













